@@ -1,5 +1,5 @@
 /* modrdn.c - bdb backend modrdn routine */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modrdn.c,v 1.63.2.13 2003/02/09 16:31:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/modrdn.c,v 1.63.2.15 2003/02/28 04:48:07 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -108,6 +108,7 @@ retry:	/* transaction retry */
 		rc = TXN_ABORT( ltid );
 		ltid = NULL;
 		op->o_private = NULL;
+		op->o_do_not_cache = opinfo.boi_acl_cache;
 		if( rc != 0 ) {
 			rc = LDAP_OTHER;
 			text = "internal error";
@@ -141,6 +142,7 @@ retry:	/* transaction retry */
 	opinfo.boi_txn = ltid;
 	opinfo.boi_locker = locker;
 	opinfo.boi_err = 0;
+	opinfo.boi_acl_cache = op->o_do_not_cache;
 	op->o_private = &opinfo;
 
 	/* get entry */
@@ -191,13 +193,13 @@ retry:	/* transaction retry */
 	/* check write on old entry */
 	rc = access_allowed( be, conn, op, e, entry, NULL, ACL_WRITE, NULL );
 
-	switch( opinfo.boi_err ) {
-	case DB_LOCK_DEADLOCK:
-	case DB_LOCK_NOTGRANTED:
-		goto retry;
-	}
-
 	if ( ! rc ) {
+		switch( opinfo.boi_err ) {
+		case DB_LOCK_DEADLOCK:
+		case DB_LOCK_NOTGRANTED:
+			goto retry;
+		}
+
 #ifdef NEW_LOGGING
 		LDAP_LOG ( OPERATION, ERR, 
 			"==>bdb_modrdn: no access to entry\n", 0, 0, 0 );
@@ -210,6 +212,41 @@ retry:	/* transaction retry */
 		goto return_results;
 	}
 
+#ifndef BDB_HIER
+	rc = bdb_dn2id_children( be, ltid, &e->e_nname, 0 );
+	if ( rc != DB_NOTFOUND ) {
+		switch( rc ) {
+		case DB_LOCK_DEADLOCK:
+		case DB_LOCK_NOTGRANTED:
+			goto retry;
+		case 0:
+#ifdef NEW_LOGGING
+			LDAP_LOG ( OPERATION, DETAIL1, 
+				"<=- bdb_modrdn: non-leaf %s\n", dn->bv_val, 0, 0 );
+#else
+			Debug(LDAP_DEBUG_ARGS,
+				"<=- bdb_modrdn: non-leaf %s\n",
+				dn->bv_val, 0, 0);
+#endif
+			rc = LDAP_NOT_ALLOWED_ON_NONLEAF;
+			text = "subtree rename not supported";
+			break;
+		default:
+#ifdef NEW_LOGGING
+			LDAP_LOG ( OPERATION, ERR, 
+				"<=- bdb_modrdn: has_children failed %s (%d)\n",
+				db_strerror(rc), rc, 0 );
+#else
+			Debug(LDAP_DEBUG_ARGS,
+				"<=- bdb_modrdn: has_children failed: %s (%d)\n",
+				db_strerror(rc), rc, 0 );
+#endif
+			rc = LDAP_OTHER;
+			text = "internal error";
+		}
+		goto return_results;
+	}
+#endif
 	if (!manageDSAit && is_entry_referral( e ) ) {
 		/* parent is a referral, don't allow add */
 		/* parent is an alias, don't allow add */
@@ -277,6 +314,12 @@ retry:	/* transaction retry */
 			children, NULL, ACL_WRITE, NULL );
 
 		if ( ! rc ) {
+			switch( opinfo.boi_err ) {
+			case DB_LOCK_DEADLOCK:
+			case DB_LOCK_NOTGRANTED:
+				goto retry;
+			}
+
 			rc = LDAP_INSUFFICIENT_ACCESS;
 #ifdef NEW_LOGGING
 			LDAP_LOG ( OPERATION, ERR, 
@@ -329,6 +372,12 @@ retry:	/* transaction retry */
 				p = NULL;
 
 				if ( ! rc ) {
+					switch( opinfo.boi_err ) {
+					case DB_LOCK_DEADLOCK:
+					case DB_LOCK_NOTGRANTED:
+						goto retry;
+					}
+
 					rc = LDAP_INSUFFICIENT_ACCESS;
 #ifdef NEW_LOGGING
 					LDAP_LOG ( OPERATION, ERR, 
@@ -398,7 +447,7 @@ retry:	/* transaction retry */
 		/*  newSuperior == oldParent? */
 		if( dn_match( &p_ndn, nnewSuperior ) ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG( BACK_LDBM, INFO, "bdb_back_modrdn: "
+			LDAP_LOG( BACK_BDB, INFO, "bdb_back_modrdn: "
 				"new parent \"%s\" same as the old parent \"%s\"\n",
 				newSuperior->bv_val, p_dn.bv_val, 0 );
 #else
@@ -468,6 +517,12 @@ retry:	/* transaction retry */
 				NULL, ACL_WRITE, NULL );
 
 			if( ! rc ) {
+				switch( opinfo.boi_err ) {
+				case DB_LOCK_DEADLOCK:
+				case DB_LOCK_NOTGRANTED:
+					goto retry;
+				}
+
 #ifdef NEW_LOGGING
 				LDAP_LOG ( OPERATION, DETAIL1, 
 					"==>bdb_modrdn: no wr to newSup children\n", 0, 0, 0 );
@@ -531,6 +586,12 @@ retry:	/* transaction retry */
 					np = NULL;
 
 					if ( ! rc ) {
+						switch( opinfo.boi_err ) {
+						case DB_LOCK_DEADLOCK:
+						case DB_LOCK_NOTGRANTED:
+							goto retry;
+						}
+
 						rc = LDAP_INSUFFICIENT_ACCESS;
 #ifdef NEW_LOGGING
 						LDAP_LOG ( OPERATION, ERR, 
@@ -621,15 +682,6 @@ retry:	/* transaction retry */
 		goto return_results;
 	}
 
-#ifdef NEW_LOGGING
-	LDAP_LOG ( OPERATION, ERR, 
-		"bdb_modrdn: new ndn=%s does not exist\n", new_ndn.bv_val, 0, 0 );
-#else
-	Debug( LDAP_DEBUG_TRACE,
-		"bdb_modrdn: new ndn=%s does not exist\n",
-		new_ndn.bv_val, 0, 0 );
-#endif
-
 	/* Get attribute type and attribute value of our new rdn, we will
 	 * need to add that to our new entry
 	 */
@@ -745,6 +797,9 @@ retry:	/* transaction retry */
 		&text, textbuf, textlen );
 
 	if( rc != LDAP_SUCCESS ) {
+		if ( ( rc == LDAP_INSUFFICIENT_ACCESS ) && opinfo.boi_err ) {
+			rc = opinfo.boi_err;
+		}
 		switch( rc ) {
 		case DB_LOCK_DEADLOCK:
 		case DB_LOCK_NOTGRANTED:
