@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.211.2.17 2003/02/21 19:59:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/daemon.c,v 1.211.2.19 2003/03/06 19:11:23 kurt Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -1381,7 +1381,7 @@ slapd_daemon_task(
 			slap_ssf_t ssf = 0;
 			char *authid = NULL;
 #ifdef SLAPD_RLOOKUPS
-			char *hebuf = NULL;
+			char hbuf[NI_MAXHOST];
 #endif
 
 			char	*dnsname = NULL;
@@ -1561,9 +1561,9 @@ slapd_daemon_task(
 
 					if( getpeereid( s, &uid, &gid ) == 0 ) {
 						authid = ch_malloc(
-							sizeof("uidNumber=4294967295+gidNumber=4294967295,"
+							sizeof("uidnumber=4294967295+gidnumber=4294967295,"
 								"cn=peercred,cn=external,cn=auth"));
-						sprintf(authid, "uidNumber=%d+gidNumber=%d,"
+						sprintf(authid, "uidnumber=%d+gidnumber=%d,"
 							"cn=peercred,cn=external,cn=auth",
 							(int) uid, (int) gid);
 					}
@@ -1612,23 +1612,12 @@ slapd_daemon_task(
 			) {
 #ifdef SLAPD_RLOOKUPS
 				if ( use_reverse_lookup ) {
-					struct hostent he;
-					int herr;
-					struct hostent *hp = NULL;
-#  ifdef LDAP_PF_INET6
-					if ( from.sa_addr.sa_family == AF_INET6 )
-						ldap_pvt_gethostbyaddr_a(
-							(char *)&(from.sa_in6_addr.sin6_addr),
-							sizeof(from.sa_in6_addr.sin6_addr),
-							AF_INET6, &he, &hebuf,
-							&hp, &herr );
-					else
-#  endif /* LDAP_PF_INET6 */
-					ldap_pvt_gethostbyaddr_a(
-						(char *) &(from.sa_in_addr.sin_addr),
-						sizeof(from.sa_in_addr.sin_addr),
-						AF_INET, &he, &hebuf, &hp, &herr );
-					dnsname = hp ? ldap_pvt_str2lower( hp->h_name ) : NULL;
+					char *herr;
+					if (ldap_pvt_get_hname( &from, len, hbuf,
+						sizeof(hbuf), &herr ) == 0) {
+						ldap_pvt_str2lower( hbuf );
+						dnsname = hbuf;
+					}
 				}
 #else
 				dnsname = NULL;
@@ -1666,9 +1655,6 @@ slapd_daemon_task(
 				authid );
 
 			if( authid ) ch_free(authid);
-#ifdef SLAPD_RLOOKUPS
-			if( hebuf ) ldap_memfree(hebuf);
-#endif
 
 			if( id < 0 ) {
 #ifdef NEW_LOGGING
