@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.61.2.8 2003/02/10 19:22:47 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.61.2.9 2003/03/05 23:48:33 kurt Exp $ */
 /* root_dse.c - Provides the ROOT DSA-Specific Entry
  *
  * Copyright 1999-2003 The OpenLDAP Foundation.
@@ -113,7 +113,7 @@ root_dse_info(
 				return LDAP_OTHER;
 			continue;
 		}
-		if ( backends[i].be_flags & SLAP_BFLAG_GLUE_SUBORDINATE ) {
+		if ( SLAP_GLUE_SUBORDINATE( &backends[i] ) ) {
 			continue;
 		}
 		for ( j = 0; backends[i].be_suffix[j].bv_val != NULL; j++ ) {
